/*
    Project: TCGINA
    Version: V1.18
    File   : tcgsharedmem.c

    Copyright (c) 2005-2008 Author of TCGINA DLL. All rights reserved.
    Public Key Fingerprint: 294B A769 4A0A CC05 DAE6 00DD FF47 8C72 4097 67CE

    Covered by TrueCrypt License 2.3 the full text of which is contained
    in the file License.txt included in TCGINA binary and source code
    distribution archives.

    The copyright notice of this source code file and the public key
    fingerprint must be retained in full.
*/

#define _WIN32_WINNT 0x0500
#include <windows.h>
#include <sddl.h>
#include "tcgpriv.h"

#ifdef SUPPORT_SHARED_MEMORY

static PTCG_SHARED_MEMORY psm = NULL;
static HANDLE hMapFile;
static HANDLE hMutex;

#define SHARED_MEMORY_NAME TEXT("TCGINA.{D127B54A-F278-4111-894F-F04F9F49A675}")
#define MUTEX_TIMEOUT      5000
#define MAX_PROFILE_COUNT  100

PTCG_SHARED_MEMORY WINAPI TCG_InitSharedMemory(PDWORD pdwProfileCount)
{
    SECURITY_ATTRIBUTES  sa;
    PSECURITY_ATTRIBUTES psa;
    DWORD                dwSharedMemorySize;

    TC_Assert(pdwProfileCount);

    if (psm != NULL) return psm;

    if (*pdwProfileCount == 0) {
        return NULL;
    }
    if (*pdwProfileCount > MAX_PROFILE_COUNT) {
        *pdwProfileCount = MAX_PROFILE_COUNT;
    }
    dwSharedMemorySize = sizeof(TCG_SHARED_MEMORY) +
      (*pdwProfileCount - 1) * sizeof(TCG_ENCRYPTED_PROFILE_INFO);

    sa.nLength = sizeof(sa);
    sa.bInheritHandle = FALSE;
    if (ConvertStringSecurityDescriptorToSecurityDescriptor(
      TEXT("D:")
      TEXT("(A;NP;GA;;;SY)")
      TEXT(""),
      SDDL_REVISION_1, &sa.lpSecurityDescriptor, NULL)) {
        psa = &sa;
    } else {
        sa.lpSecurityDescriptor = NULL;
        psa = NULL;
    }

    hMutex = CreateMutex(psa, FALSE,
      TEXT("Global\\") SHARED_MEMORY_NAME TEXT(".Mutex"));
    if (hMutex == NULL) {
        TC_DbgPrintfA("Cannot create mutex (ErrorCode=%d)\n",
          (int) GetLastError());
        LocalFree(sa.lpSecurityDescriptor);
        return NULL;
    }
    if (WaitForSingleObject(hMutex, MUTEX_TIMEOUT) != WAIT_OBJECT_0) {
        TC_DbgPrintfA("Cannot get mutex ownership\n");
        LocalFree(sa.lpSecurityDescriptor);
        return NULL;
    }

    hMapFile = CreateFileMapping(INVALID_HANDLE_VALUE, psa, PAGE_READWRITE, 0,
      dwSharedMemorySize, TEXT("Global\\") SHARED_MEMORY_NAME);
    if (hMapFile == NULL) {
        TC_DbgPrintfA("Cannot create file mapping (ErrorCode=%d)\n",
            (int)GetLastError());
        ReleaseMutex(hMutex);
        LocalFree(sa.lpSecurityDescriptor);
        CloseHandle(hMutex);
        return NULL;
    }
    LocalFree(sa.lpSecurityDescriptor);

    psm = (PTCG_SHARED_MEMORY) MapViewOfFile(hMapFile, FILE_MAP_WRITE, 0, 0,
      dwSharedMemorySize);
    if (psm == NULL) {
        TC_DbgPrintfA("MapViewOfFile failed (ErrorCode=%d)\n",
          GetLastError());
        ReleaseMutex(hMutex);
        CloseHandle(hMapFile);
        CloseHandle(hMutex);
        return NULL;
    }
    if (psm->dwProfileCount == 0) {
        psm->dwProfileCount = *pdwProfileCount;
    } else {
        *pdwProfileCount = psm->dwProfileCount;
    }

    ReleaseMutex(hMutex);

    return psm;
}

BOOL WINAPI TCG_LockSharedMemory(void)
{
    if (psm == NULL) {
        return FALSE;
    }

    if (WaitForSingleObject(hMutex, MUTEX_TIMEOUT) != WAIT_OBJECT_0) {
        TC_DbgPrintfA("Cannot lock shared memory!\n");
        return FALSE;
    }

    return TRUE;
}

BOOL WINAPI TCG_UnlockSharedMemory(void)
{
    if (psm == NULL) {
        return FALSE;
    }

    return ReleaseMutex(hMutex);
}

#if 0
VOID WINAPI TCG_FreeSharedMemory(void)
{
    if (psm) {
        UnmapViewOfFile((LPCVOID)psm);
        CloseHandle(hMapFile);
        CloseHandle(hMutex);
        psm = NULL;
    }
}
#endif

#endif
